/*
 * *********************************************************************************************************************
 *
 *          !!
 *        .F88X
 *        X8888Y
 *      .}888888N;
 *        i888888N;        .:!              .I$WI:
 *          R888888I      .'N88~            i8}+8Y&8"l8i$8>8W~'>W8}8]KW+8IIN"8&
 *          .R888888I    .;N8888~          .X8'  "8I.!,/8"  !%NY8`"8I8~~8>,88I
 *            +888888N;  .8888888Y                                  "&&8Y.}8,
 *            ./888888N;  .R888888Y        .'}~    .>}'.`+>  i}!    "i'  +/'  .'i~  !11,.:">,  .~]!  .i}i
 *              ~888888%:  .I888888l      .]88~`1/iY88Ii+1'.R$8$8]"888888888>  Y8$  W8E  X8E  W8888'188Il}Y88$*
 *              18888888    E8888881    .]W%8$`R8X'&8%++N8i,8N%N8+l8%`  .}8N:.R$RE%N88N%N$K$R  188,FE$8%~Y88I
 *            .E888888I  .i8888888'      .:$8I;88+`E8R:/8N,.>881.`$8E/1/]N8X.Y8N`"KF&&FK!'88*."88K./$88%RN888+~
 *            8888888I  .,N888888~        ~88i"8W,!N8*.I88.}888%F,i$88"F88"  888:E8X.>88!i88>`888*.}Fl1]*}1YKi'
 *          i888888N'      I888Y          ]88;/EX*IFKFK88X  K8R  .l8W  88Y  ~88}'88E&%8W.X8N``]88!.$8K  .:W8I
 *        .i888888N;        I8Y          .&8$  .X88!  i881.:%888>I88  ;88]  +88+.';;;;:.Y88X  18N.,88l  .+88/
 *      .:R888888I
 *      .&888888I                                          Copyright (c) 2009-2020.  博锐尚格科技股份有限公司
 *        ~8888'
 *        .!88~                                                                     All rights reserved.
 *
 * *********************************************************************************************************************
 */

package com.persagy.server.controllers.rel

import com.persagy.base.extensions.toJson
import com.persagy.server.datacenter.models.entities.rel.REqForFl
import com.persagy.server.datacenter.models.entities.rel.REqForFlList
import com.persagy.server.datacenter.models.entities.rel.RFlForEqList
import com.persagy.server.services.rel.REqForFlService
import com.persagy.service.models.responses.SBaseResponse
import com.persagy.service.models.responses.SCreateResponse
import io.swagger.v3.oas.annotations.Operation
import io.swagger.v3.oas.annotations.tags.Tag
import org.slf4j.LoggerFactory
import org.springframework.web.bind.annotation.PostMapping
import org.springframework.web.bind.annotation.RequestBody
import org.springframework.web.bind.annotation.RequestMapping
import org.springframework.web.bind.annotation.RestController

/**
 * 设备服务于楼层
 *
 * @author 付国宝
 */
@Tag(name = "037、关系-设备服务于楼层")
@RestController
@RequestMapping("/rel/eq-for-fl")
class REqForFlController {
    companion object {
        /** 日志 */
        private val logger = LoggerFactory.getLogger(REqForFlController::class.java)
    } // Companion object

    /**
     * 创建设备服务于楼层关系
     *
     * @param   rEqForFl     设备和楼层的关系对象
     * @return               创建的结果信息
     */
    @Operation(summary = "创建设备服务于楼层关系", description = "必填项：EquipId（设备id）、FloorId（楼层id）")
    @PostMapping(value = ["/link"])
    fun create(@RequestBody rEqForFl: REqForFl): SCreateResponse<REqForFl> {
        logger.debug("floor = ${rEqForFl.toJson()}")
        return REqForFlService.create(rEqForFl)
    } // Function create()

    /**
     * 根据对象删除关系
     *
     * @param   rEqForFl      对象
     * @return                删除的结果信息
     */
    @Operation(summary = "根据对象删除设备服务于楼层的关系，只针对一个对象", description = "必填项：EquipId（设备id）、FloorId(楼层id)")
    @PostMapping(value = ["/unlink"])
    fun delete(@RequestBody rEqForFl: REqForFl): SBaseResponse {
        return REqForFlService.deletes(rEqForFl)
    } // Function delete()

    /**
     * 创建设备服务于楼层关系
     *
     * @param   rEqForFl    设备和楼层关系对象
     * @return              创建的结果信息
     */
    @Operation(summary = "设备服务于楼层，设备一对多", description = "必填项：FloorIdList（楼层id列表）、EquipId（设备id）,测方法会覆盖以前的记录")
    @PostMapping(value = ["link-fl"])
    fun createFl(@RequestBody rEqForFl: REqForFlList): SCreateResponse<REqForFl>{
        logger.debug("floor =${rEqForFl.toJson()}")
        return REqForFlService.createEq(rEqForFl)
    } // Function createFl()

    /**
     * 创建服务于楼层的设备关系
     *
     * @param   rEqForFl    设备和楼层关系对象
     * @return              创建的结果信息
     */
    @Operation(summary = "服务于楼层的设备，楼层一对多",description = "必填项：EquipIdList（设备id列表）、FloorId（建筑id），测方法会覆盖以前的记录")
    @PostMapping(value = ["link-eq"])
    fun createEq(@RequestBody rEqForFl: RFlForEqList): SCreateResponse<REqForFl>{
        logger.debug("floor =${rEqForFl.toJson()}")
        return REqForFlService.createsFl(rEqForFl)
    } // Function createSy()

} // Class REqForFlController